from openapi.api.meters_api import MetersApi
from vayu_sdk.clients.vayu_client import VayuClient
from openapi.models.update_meter_request import UpdateMeterRequest

from openapi.models.aggregation_operator import AggregationOperator
from openapi.models.aggregation_method import AggregationMethod
from openapi.models.filter import Filter
from openapi.models.condition import Condition
from openapi.models.criterion import Criterion
from openapi.models.criterion_operator import CriterionOperator

from openapi.models.meter import Meter
from openapi.models.list_meters_response import ListMetersResponse
from openapi.models.get_meter_response import GetMeterResponse
from openapi.models.update_meter_response import UpdateMeterResponse
from openapi.models.delete_meter_response import DeleteMeterResponse


class MetersAPI:
    __client: MetersApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = MetersApi(vayu_client.client)

    def list(self, limit: int = None, cursor: int = None):
        return self.__client.list_meters(limit=limit, cursor=cursor)

    def get(self, id: str):
        response = self.__client.get_meter(meter_id=id)

        return response

    def update(
        self,
        id: str,
        name: str = None,
        event_name: str = None,
        aggregation_method: AggregationMethod = None,
        filter: Filter = None,
    ):
        request = UpdateMeterRequest(
            name=name,
            eventName=event_name,
            aggregationMethod=aggregation_method,
            filter=filter,
        )

        response = self.__client.update_meter(meter_id=id, update_meter_request=request)

        return response

    def delete(self, id: str):
        response = self.__client.delete_meter(meter_id=id)

        return response



from functools import singledispatch
from typing import List
from openapi.api.customers_api import CustomersApi
from openapi.models.contact import Contact
from vayu_sdk.clients.vayu_client import VayuClient
from openapi.models.create_customer_request import CreateCustomerRequest
from openapi.models.update_customer_request import UpdateCustomerRequest

from openapi.models.create_customer_response_customer import (
    CreateCustomerResponseCustomer,
)
from openapi.models.list_customers_response import ListCustomersResponse
from openapi.models.get_customer_response import GetCustomerResponse
from openapi.models.create_customer_response import CreateCustomerResponse
from openapi.models.update_customer_response import UpdateCustomerResponse
from openapi.models.delete_customer_response import DeleteCustomerResponse
from openapi.models.address import Address


Customer = CreateCustomerResponseCustomer


class CustomersAPI:
    __client: CustomersApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = CustomersApi(vayu_client.client)

    def list(self, cursor: str = None, limit: int = None):
        return self.__client.list_customers(limit=limit, cursor=cursor)

    def get(self, id: str):
        response = self.__client.get_customer(customer_id=id)

        return response
    
    def getByExternalId(self, external_id: str):
        response = self.__client.get_customer_by_external_id(external_id=external_id)

        return response

    def create(
        self, name: str, external_id: str, aliases: List[str] = [], address: Address = None, contacts: List[Contact] = None
    ):
        request = CreateCustomerRequest(
            name=name, externalId=external_id, aliases=aliases, address=address, contacts=contacts
        )
        response = self.__client.create_customer(create_customer_request=request)
        return response

    def update(self, id: str, name: str = None, external_id: str = None, aliases: List[str] = None, address: Address = None, contacts: List[Contact] = None):
        request = UpdateCustomerRequest(name=name, externalId=external_id, address=address, contacts=contacts, aliases=aliases)
        response = self.__client.update_customer(customer_id=id, update_customer_request=request)

        return response

    def delete(self, id: str):
        response = self.__client.delete_customer(customer_id=id)

        return response



from openapi.api.webhooks_api import WebhooksApi
from openapi.models.notification_event_type import NotificationEventType
from openapi.models.webhook_subscribe_request import WebhookSubscribeRequest
from vayu_sdk.clients.vayu_client import VayuClient

class WebhooksAPI:
    __client: WebhooksApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = WebhooksApi(vayu_client.client)

    def subscribe(self, callback_url: str, event_type: NotificationEventType):
        request = WebhookSubscribeRequest(callback_url=callback_url, event_type=event_type)
        return self.__client.webhook_subscribe(webhook_subscribe_request=request)



from datetime import datetime
from typing import List

from openapi.api.events_api import EventsApi
from openapi.models.delete_event_response import DeleteEventResponse
from openapi.models.event import Event
from openapi.models.events_dry_run_request import EventsDryRunRequest
from openapi.models.events_dry_run_response import EventsDryRunResponse
from openapi.models.get_event_response import GetEventResponse
from openapi.models.query_events_response import QueryEventsResponse
from openapi.models.send_events_request import SendEventsRequest
from openapi.models.send_events_response import SendEventsResponse
from vayu_sdk.clients.vayu_client import VayuClient


class EventsAPI:
    __client: EventsApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = EventsApi(vayu_client.client)

    def get(self, ref: str):
        response = self.__client.get_event_by_ref_id(ref_id=ref)

        return response

    def delete(self, ref: str):
        response = self.__client.delete_event_by_ref_id(ref_id=ref)

        return response

    def query(
        self,
        start_time: datetime,
        end_time: datetime,
        event_name: str,
        cursor: str = None,
        limit: int = None,
    ):
        response = self.__client.query_events(
            start_time=start_time,
            end_time=end_time,
            event_name=event_name,
            cursor=cursor,
            limit=limit,
        )

        return response

    def send(self, events: List[Event]):
        request = SendEventsRequest(events=events)

        return self.__client.send_events(send_events_request=request)

    def dry_run(self, events: List[Event]):
        request = EventsDryRunRequest(events=events)

        return self.__client.send_events_dry_run(events_dry_run_request=request)




from openapi.api.invoices_api import InvoicesApi
from vayu_sdk.clients.vayu_client import VayuClient

from openapi.models.get_invoice_response_invoice import GetInvoiceResponseInvoice
from openapi.models.list_invoices_response import ListInvoicesResponse
from openapi.models.get_invoice_response import GetInvoiceResponse

Invoice = GetInvoiceResponseInvoice

class InvoicesAPI:
    __client: InvoicesApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = InvoicesApi(vayu_client.client)
    
    def list(self, limit: int = None, cursor: int = None):
        return self.__client.list_invoices(limit=limit, cursor=cursor)

    def get(self, id: str):
        get_invoice_response = self.__client.get_invoice(invoice_id=id)

        return get_invoice_response



from openapi.api.contracts_api import ContractsApi
from vayu_sdk.clients.vayu_client import VayuClient
from openapi.models.create_contract_request import CreateContractRequest
from datetime import datetime

from openapi.models.get_contract_response_contract import GetContractResponseContract
from openapi.models.list_contracts_response import ListContractsResponse
from openapi.models.get_contract_response import GetContractResponse
from openapi.models.create_contract_response import CreateContractResponse
from openapi.models.delete_contract_response import DeleteContractResponse

Contract = GetContractResponseContract


class ContractsAPI:
    __client: ContractsApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = ContractsApi(vayu_client.client)

    def list(self, limit: int = None, cursor: int = None):
        return self.__client.list_contracts(limit=limit, cursor=cursor)

    def get(self, id: str):
        response = self.__client.get_contract(contract_id=id)

        return response

    def create(
        self, start_date: datetime, end_date: datetime | None, customer_id: str, plan_id: str
    ):
        request = CreateContractRequest(
            startDate=start_date,
            endDate=end_date,
            customerId=customer_id,
            planId=plan_id,
        )

        response = self.__client.create_contract(
            create_contract_request=request
        )

        return response

    def delete(self, id: str):
        response = self.__client.delete_contract(contract_id=id)

        return response



# flake8: noqa

# import apis into api package
from openapi.api.auth_api import AuthApi
from openapi.api.contracts_api import ContractsApi
from openapi.api.credits_api import CreditsApi
from openapi.api.customers_api import CustomersApi
from openapi.api.events_api import EventsApi
from openapi.api.integrations_api import IntegrationsApi
from openapi.api.invoices_api import InvoicesApi
from openapi.api.meters_api import MetersApi
from openapi.api.plans_api import PlansApi
from openapi.api.reports_api import ReportsApi
from openapi.api.webhooks_api import WebhooksApi




from openapi.api_client import ApiClient
from openapi.configuration import Configuration
from openapi.api.auth_api import AuthApi
from openapi.models.login_request import LoginRequest
import jwt
from datetime import datetime, timedelta
from openapi.rest import RESTClientObject


class VayuClient:
    __access_token: str = None
    __expires_at: datetime = None
    __api_key: str = None
    __host: str = None
    __token_expiry_threshold = timedelta(minutes=5)

    def __init__(self, api_key: str, host: str = None):
        self.__access_token = None
        self.__host = host
        self.__api_key = api_key

    @property
    def access_token(self):
        return self.__access_token

    @property
    def is_logged_in(self):
        return self.__access_token is not None

    @property
    def client(self):
        configuration = Configuration(host=self.__host, api_key=self.__access_token)
        client = ApiClient(configuration)
        client.rest_client = AutoTokenRefresherRESTClient(self, configuration)
        return client

    def login(self):
        configuration = Configuration(host=self.__host)
        client = ApiClient(configuration)
        api = AuthApi(client)
        login_request = LoginRequest(refreshToken=self.__api_key)
        refresh_response = api.login(login_request)

        self.__access_token = refresh_response.access_token

        decoded_token = jwt.decode(
            self.__access_token, options={"verify_signature": False}
        )
        self.__expires_at = datetime.fromtimestamp(decoded_token["exp"])

    def refresh_token(self):
        if (
            self.__access_token is None
            or datetime.now() + self.__token_expiry_threshold > self.__expires_at
        ):
            
            self.login()

    def validate_logged_in(self):
        if not self.is_logged_in:
            raise RuntimeError(
                "Vayu client is not logged in. Please login before calling this method"
            )


class AutoTokenRefresherRESTClient(RESTClientObject):
    def __init__(self, vayu_client: VayuClient, configuration: Configuration, *args, **kwargs):
        super().__init__(configuration, *args, **kwargs)
        self.vayu_client = vayu_client

    def request(self, method, url, headers=None, **kwargs):
        if "/login" not in url:
            self.vayu_client.refresh_token()
            if headers is None:
                headers = {}
            headers["Authorization"] = f"Bearer {self.vayu_client.access_token}"

        return super().request(method, url, headers=headers, **kwargs)


__all__ = ["VayuClient"]



from .api_webhooks import (
  WebhooksAPI,
  WebhookSubscribeRequest,
  NotificationEventType,
)

from .api_contracts import (
  Contract,
  ContractsAPI,
  CreateContractRequest,
  CreateContractResponse,
  DeleteContractResponse,
  GetContractResponse,
  GetContractResponseContract,
  ListContractsResponse,
)

from .api_customers import (
  Address,
  Contact,
  CreateCustomerRequest,
  CreateCustomerResponse,
  CreateCustomerResponseCustomer,
  CustomersAPI,
  DeleteCustomerResponse,
  GetCustomerResponse,
  ListCustomersResponse,
  UpdateCustomerRequest,
  UpdateCustomerResponse,
)

from .api_events import (
  Event,
  EventsAPI,
  GetEventResponse,
  EventsDryRunResponse,
  DeleteEventResponse,
  QueryEventsResponse,
  SendEventsRequest,
  SendEventsResponse,
)

from .api_invoices import (
  InvoicesAPI,
  Invoice,
  GetInvoiceResponse,
  GetInvoiceResponseInvoice,
  ListInvoicesResponse,
)

from .api_meters import (
  MetersAPI,
  Meter,
  GetMeterResponse,
  AggregationMethod,
  AggregationOperator,
  Condition,
  Criterion,
  CriterionOperator,
  DeleteMeterResponse,
  Filter,
  ListMetersResponse,
  UpdateMeterRequest,
  UpdateMeterResponse,
)

from .api_plans import (
  Plan,
  PlansAPI,
  DeletePlanResponse,
  GetPlanResponse,
  GetPlanResponsePlan,
  ListPlansResponse,
)


from .clients import VayuClient
from .apis import * 

class Vayu:
    __api_key: str = None
    __host: str = None
    __client: VayuClient = None

    def __init__(self, api_key: str, host: str = None):
        self.__client = None
        self.__host = host
        self.__api_key = api_key

    @property
    def contracts(self) -> ContractsAPI:
        return ContractsAPI(self.__client)

    @property
    def customers(self) -> CustomersAPI:
        return CustomersAPI(self.__client)

    @property
    def events(self) -> EventsAPI:
        return EventsAPI(self.__client)

    @property
    def invoices(self) -> InvoicesAPI:
        return InvoicesAPI(self.__client)

    @property
    def meters(self) -> MetersAPI:
        return MetersAPI(self.__client)

    @property
    def plans(self) -> PlansAPI:
        return PlansAPI(self.__client)
    
    @property
    def webhooks(self) -> WebhooksAPI:
        return WebhooksAPI(self.__client)
    
    def login(self):
        self.__client = VayuClient(self.__api_key, self.__host)
        self.__client.login()



# coding: utf-8

# flake8: noqa
"""
    Vayu API

    The Vayu API is a RESTful API that allows you to submit events for processing and storage & manage billing related entities.           The API is secured using the Bearer Authentication scheme with JWT tokens.           To obtain a JWT token, please contact Vayu at team@withvayu.com

    The version of the OpenAPI document: 1.0.0
    Contact: dev@withvayu.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi.models.address import Address
from openapi.models.aggregation_method import AggregationMethod
from openapi.models.aggregation_operator import AggregationOperator
from openapi.models.billing_cycle_status import BillingCycleStatus
from openapi.models.billing_interval import BillingInterval
from openapi.models.condition import Condition
from openapi.models.contact import Contact
from openapi.models.contract_status import ContractStatus
from openapi.models.create_contract_request import CreateContractRequest
from openapi.models.create_contract_response import CreateContractResponse
from openapi.models.create_customer_request import CreateCustomerRequest
from openapi.models.create_customer_response import CreateCustomerResponse
from openapi.models.create_customer_response_customer import CreateCustomerResponseCustomer
from openapi.models.credit_ledger_entry import CreditLedgerEntry
from openapi.models.criterion import Criterion
from openapi.models.criterion_operator import CriterionOperator
from openapi.models.currency import Currency
from openapi.models.deduct_credits_request import DeductCreditsRequest
from openapi.models.delete_contract_response import DeleteContractResponse
from openapi.models.delete_contract_response_contract import DeleteContractResponseContract
from openapi.models.delete_customer_response import DeleteCustomerResponse
from openapi.models.delete_customer_response_customer import DeleteCustomerResponseCustomer
from openapi.models.delete_event_response import DeleteEventResponse
from openapi.models.delete_event_response_event import DeleteEventResponseEvent
from openapi.models.delete_meter_response import DeleteMeterResponse
from openapi.models.delete_meter_response_meter import DeleteMeterResponseMeter
from openapi.models.delete_plan_response import DeletePlanResponse
from openapi.models.delete_plan_response_plan import DeletePlanResponsePlan
from openapi.models.event import Event
from openapi.models.events_dry_run_request import EventsDryRunRequest
from openapi.models.events_dry_run_response import EventsDryRunResponse
from openapi.models.events_dry_run_response_object import EventsDryRunResponseObject
from openapi.models.events_dry_run_response_object_event import EventsDryRunResponseObjectEvent
from openapi.models.events_dry_run_response_object_meter_with_values_inner import EventsDryRunResponseObjectMeterWithValuesInner
from openapi.models.filter import Filter
from openapi.models.full_day_period import FullDayPeriod
from openapi.models.get_commitment_report_response import GetCommitmentReportResponse
from openapi.models.get_contract_response import GetContractResponse
from openapi.models.get_contract_response_contract import GetContractResponseContract
from openapi.models.get_customer_response import GetCustomerResponse
from openapi.models.get_event_response import GetEventResponse
from openapi.models.get_event_response_event import GetEventResponseEvent
from openapi.models.get_invoice_response import GetInvoiceResponse
from openapi.models.get_invoice_response_invoice import GetInvoiceResponseInvoice
from openapi.models.get_meter_response import GetMeterResponse
from openapi.models.get_meter_response_meter import GetMeterResponseMeter
from openapi.models.get_plan_response import GetPlanResponse
from openapi.models.get_plan_response_plan import GetPlanResponsePlan
from openapi.models.get_products_usage_report_response import GetProductsUsageReportResponse
from openapi.models.grant_credits_request import GrantCreditsRequest
from openapi.models.invalid_event import InvalidEvent
from openapi.models.line_item import LineItem
from openapi.models.line_item_revenue_breakdown import LineItemRevenueBreakdown
from openapi.models.list_contracts_response import ListContractsResponse
from openapi.models.list_credit_ledger_entries_response import ListCreditLedgerEntriesResponse
from openapi.models.list_customers_response import ListCustomersResponse
from openapi.models.list_invoices_response import ListInvoicesResponse
from openapi.models.list_meters_response import ListMetersResponse
from openapi.models.list_plans_response import ListPlansResponse
from openapi.models.login_request import LoginRequest
from openapi.models.login_response import LoginResponse
from openapi.models.meter import Meter
from openapi.models.net_suite_export_sales_order_request import NetSuiteExportSalesOrderRequest
from openapi.models.net_suite_sync_invoices_request import NetSuiteSyncInvoicesRequest
from openapi.models.net_suite_sync_invoices_request_data import NetSuiteSyncInvoicesRequestData
from openapi.models.net_suite_sync_invoices_request_data_entity import NetSuiteSyncInvoicesRequestDataEntity
from openapi.models.net_suite_sync_invoices_request_data_item import NetSuiteSyncInvoicesRequestDataItem
from openapi.models.net_suite_sync_invoices_request_data_item_items_inner import NetSuiteSyncInvoicesRequestDataItemItemsInner
from openapi.models.net_suite_sync_invoices_response import NetSuiteSyncInvoicesResponse
from openapi.models.notification_event_type import NotificationEventType
from openapi.models.payment_term import PaymentTerm
from openapi.models.plan_billing_data import PlanBillingData
from openapi.models.plan_status import PlanStatus
from openapi.models.query_events_response import QueryEventsResponse
from openapi.models.query_events_response_events_inner import QueryEventsResponseEventsInner
from openapi.models.send_events_request import SendEventsRequest
from openapi.models.send_events_response import SendEventsResponse
from openapi.models.update_customer_request import UpdateCustomerRequest
from openapi.models.update_customer_response import UpdateCustomerResponse
from openapi.models.update_meter_request import UpdateMeterRequest
from openapi.models.update_meter_response import UpdateMeterResponse
from openapi.models.webhook_subscribe_request import WebhookSubscribeRequest



# coding: utf-8

# flake8: noqa

"""
    Vayu API

    The Vayu API is a RESTful API that allows you to submit events for processing and storage & manage billing related entities.           The API is secured using the Bearer Authentication scheme with JWT tokens.           To obtain a JWT token, please contact Vayu at team@withvayu.com

    The version of the OpenAPI document: 1.0.0
    Contact: dev@withvayu.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from openapi.api.auth_api import AuthApi
from openapi.api.contracts_api import ContractsApi
from openapi.api.credits_api import CreditsApi
from openapi.api.customers_api import CustomersApi
from openapi.api.events_api import EventsApi
from openapi.api.integrations_api import IntegrationsApi
from openapi.api.invoices_api import InvoicesApi
from openapi.api.meters_api import MetersApi
from openapi.api.plans_api import PlansApi
from openapi.api.reports_api import ReportsApi
from openapi.api.webhooks_api import WebhooksApi

# import ApiClient
from openapi.api_response import ApiResponse
from openapi.api_client import ApiClient
from openapi.configuration import Configuration
from openapi.exceptions import OpenApiException
from openapi.exceptions import ApiTypeError
from openapi.exceptions import ApiValueError
from openapi.exceptions import ApiKeyError
from openapi.exceptions import ApiAttributeError
from openapi.exceptions import ApiException

# import models into sdk package
from openapi.models.address import Address
from openapi.models.aggregation_method import AggregationMethod
from openapi.models.aggregation_operator import AggregationOperator
from openapi.models.billing_cycle_status import BillingCycleStatus
from openapi.models.billing_interval import BillingInterval
from openapi.models.condition import Condition
from openapi.models.contact import Contact
from openapi.models.contract_status import ContractStatus
from openapi.models.create_contract_request import CreateContractRequest
from openapi.models.create_contract_response import CreateContractResponse
from openapi.models.create_customer_request import CreateCustomerRequest
from openapi.models.create_customer_response import CreateCustomerResponse
from openapi.models.create_customer_response_customer import CreateCustomerResponseCustomer
from openapi.models.credit_ledger_entry import CreditLedgerEntry
from openapi.models.criterion import Criterion
from openapi.models.criterion_operator import CriterionOperator
from openapi.models.currency import Currency
from openapi.models.deduct_credits_request import DeductCreditsRequest
from openapi.models.delete_contract_response import DeleteContractResponse
from openapi.models.delete_contract_response_contract import DeleteContractResponseContract
from openapi.models.delete_customer_response import DeleteCustomerResponse
from openapi.models.delete_customer_response_customer import DeleteCustomerResponseCustomer
from openapi.models.delete_event_response import DeleteEventResponse
from openapi.models.delete_event_response_event import DeleteEventResponseEvent
from openapi.models.delete_meter_response import DeleteMeterResponse
from openapi.models.delete_meter_response_meter import DeleteMeterResponseMeter
from openapi.models.delete_plan_response import DeletePlanResponse
from openapi.models.delete_plan_response_plan import DeletePlanResponsePlan
from openapi.models.event import Event
from openapi.models.events_dry_run_request import EventsDryRunRequest
from openapi.models.events_dry_run_response import EventsDryRunResponse
from openapi.models.events_dry_run_response_object import EventsDryRunResponseObject
from openapi.models.events_dry_run_response_object_event import EventsDryRunResponseObjectEvent
from openapi.models.events_dry_run_response_object_meter_with_values_inner import EventsDryRunResponseObjectMeterWithValuesInner
from openapi.models.filter import Filter
from openapi.models.full_day_period import FullDayPeriod
from openapi.models.get_commitment_report_response import GetCommitmentReportResponse
from openapi.models.get_contract_response import GetContractResponse
from openapi.models.get_contract_response_contract import GetContractResponseContract
from openapi.models.get_customer_response import GetCustomerResponse
from openapi.models.get_event_response import GetEventResponse
from openapi.models.get_event_response_event import GetEventResponseEvent
from openapi.models.get_invoice_response import GetInvoiceResponse
from openapi.models.get_invoice_response_invoice import GetInvoiceResponseInvoice
from openapi.models.get_meter_response import GetMeterResponse
from openapi.models.get_meter_response_meter import GetMeterResponseMeter
from openapi.models.get_plan_response import GetPlanResponse
from openapi.models.get_plan_response_plan import GetPlanResponsePlan
from openapi.models.get_products_usage_report_response import GetProductsUsageReportResponse
from openapi.models.grant_credits_request import GrantCreditsRequest
from openapi.models.invalid_event import InvalidEvent
from openapi.models.line_item import LineItem
from openapi.models.line_item_revenue_breakdown import LineItemRevenueBreakdown
from openapi.models.list_contracts_response import ListContractsResponse
from openapi.models.list_credit_ledger_entries_response import ListCreditLedgerEntriesResponse
from openapi.models.list_customers_response import ListCustomersResponse
from openapi.models.list_invoices_response import ListInvoicesResponse
from openapi.models.list_meters_response import ListMetersResponse
from openapi.models.list_plans_response import ListPlansResponse
from openapi.models.login_request import LoginRequest
from openapi.models.login_response import LoginResponse
from openapi.models.meter import Meter
from openapi.models.net_suite_export_sales_order_request import NetSuiteExportSalesOrderRequest
from openapi.models.net_suite_sync_invoices_request import NetSuiteSyncInvoicesRequest
from openapi.models.net_suite_sync_invoices_request_data import NetSuiteSyncInvoicesRequestData
from openapi.models.net_suite_sync_invoices_request_data_entity import NetSuiteSyncInvoicesRequestDataEntity
from openapi.models.net_suite_sync_invoices_request_data_item import NetSuiteSyncInvoicesRequestDataItem
from openapi.models.net_suite_sync_invoices_request_data_item_items_inner import NetSuiteSyncInvoicesRequestDataItemItemsInner
from openapi.models.net_suite_sync_invoices_response import NetSuiteSyncInvoicesResponse
from openapi.models.notification_event_type import NotificationEventType
from openapi.models.payment_term import PaymentTerm
from openapi.models.plan_billing_data import PlanBillingData
from openapi.models.plan_status import PlanStatus
from openapi.models.query_events_response import QueryEventsResponse
from openapi.models.query_events_response_events_inner import QueryEventsResponseEventsInner
from openapi.models.send_events_request import SendEventsRequest
from openapi.models.send_events_response import SendEventsResponse
from openapi.models.update_customer_request import UpdateCustomerRequest
from openapi.models.update_customer_response import UpdateCustomerResponse
from openapi.models.update_meter_request import UpdateMeterRequest
from openapi.models.update_meter_response import UpdateMeterResponse
from openapi.models.webhook_subscribe_request import WebhookSubscribeRequest



# coding: utf-8

"""
    Vayu API

    The Vayu API is a RESTful API that allows you to submit events for processing and storage & manage billing related entities.           The API is secured using the Bearer Authentication scheme with JWT tokens.           To obtain a JWT token, please contact Vayu at team@withvayu.com

    The version of the OpenAPI document: 1.0.0
    Contact: dev@withvayu.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr, field_validator
from typing import Optional, Union
from typing_extensions import Annotated
from openapi.models.deduct_credits_request import DeductCreditsRequest
from openapi.models.grant_credits_request import GrantCreditsRequest
from openapi.models.list_credit_ledger_entries_response import ListCreditLedgerEntriesResponse

from openapi.api_client import ApiClient, RequestSerialized
from openapi.api_response import ApiResponse
from openapi.rest import RESTResponseType


class CreditsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def deduct_credits(
        self,
        deduct_credits_request: DeductCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Deduct credits from customer credit ledger

        This endpoint is used to deduct credits from the specified customer account.      Submit the credit amount and customer ID in the request body to deduct the credits.      The deducted credits will be subtracted from the customer's account.

        :param deduct_credits_request: (required)
        :type deduct_credits_request: DeductCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deduct_credits_serialize(
            deduct_credits_request=deduct_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def deduct_credits_with_http_info(
        self,
        deduct_credits_request: DeductCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Deduct credits from customer credit ledger

        This endpoint is used to deduct credits from the specified customer account.      Submit the credit amount and customer ID in the request body to deduct the credits.      The deducted credits will be subtracted from the customer's account.

        :param deduct_credits_request: (required)
        :type deduct_credits_request: DeductCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deduct_credits_serialize(
            deduct_credits_request=deduct_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def deduct_credits_without_preload_content(
        self,
        deduct_credits_request: DeductCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Deduct credits from customer credit ledger

        This endpoint is used to deduct credits from the specified customer account.      Submit the credit amount and customer ID in the request body to deduct the credits.      The deducted credits will be subtracted from the customer's account.

        :param deduct_credits_request: (required)
        :type deduct_credits_request: DeductCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._deduct_credits_serialize(
            deduct_credits_request=deduct_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _deduct_credits_serialize(
        self,
        deduct_credits_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if deduct_credits_request is not None:
            _body_params = deduct_credits_request



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/credits/deduct',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def grant_credits(
        self,
        grant_credits_request: GrantCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Grant credits to a customer credit ledger

        This endpoint is used to grant credits to a customer.     Submit the credit amount and customer ID in the request body to grant the credits.     The granted credits will be added to the customer's account.

        :param grant_credits_request: (required)
        :type grant_credits_request: GrantCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_credits_serialize(
            grant_credits_request=grant_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def grant_credits_with_http_info(
        self,
        grant_credits_request: GrantCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Grant credits to a customer credit ledger

        This endpoint is used to grant credits to a customer.     Submit the credit amount and customer ID in the request body to grant the credits.     The granted credits will be added to the customer's account.

        :param grant_credits_request: (required)
        :type grant_credits_request: GrantCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_credits_serialize(
            grant_credits_request=grant_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def grant_credits_without_preload_content(
        self,
        grant_credits_request: GrantCreditsRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Grant credits to a customer credit ledger

        This endpoint is used to grant credits to a customer.     Submit the credit amount and customer ID in the request body to grant the credits.     The granted credits will be added to the customer's account.

        :param grant_credits_request: (required)
        :type grant_credits_request: GrantCreditsRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._grant_credits_serialize(
            grant_credits_request=grant_credits_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _grant_credits_serialize(
        self,
        grant_credits_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if grant_credits_request is not None:
            _body_params = grant_credits_request



        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/credits/grant',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_credit_ledger_entries(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ListCreditLedgerEntriesResponse:
        """Retrieve credit ledger entries for customer.

        This endpoint is used to retrieve the credit ledger entries for a specific customer.

        :param customer_id: (required)
        :type customer_id: str
        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_credit_ledger_entries_serialize(
            customer_id=customer_id,
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCreditLedgerEntriesResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_credit_ledger_entries_with_http_info(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ListCreditLedgerEntriesResponse]:
        """Retrieve credit ledger entries for customer.

        This endpoint is used to retrieve the credit ledger entries for a specific customer.

        :param customer_id: (required)
        :type customer_id: str
        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_credit_ledger_entries_serialize(
            customer_id=customer_id,
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCreditLedgerEntriesResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_credit_ledger_entries_without_preload_content(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve credit ledger entries for customer.

        This endpoint is used to retrieve the credit ledger entries for a specific customer.

        :param customer_id: (required)
        :type customer_id: str
        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_credit_ledger_entries_serialize(
            customer_id=customer_id,
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCreditLedgerEntriesResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_credit_ledger_entries_serialize(
        self,
        customer_id,
        limit,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if customer_id is not None:
            
            _query_params.append(('customerId', customer_id))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/credits',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





from openapi.api.plans_api import PlansApi
from vayu_sdk.clients.vayu_client import VayuClient

from openapi.models.get_plan_response_plan import GetPlanResponsePlan
from openapi.models.list_plans_response import ListPlansResponse
from openapi.models.get_plan_response import GetPlanResponse
from openapi.models.delete_plan_response import DeletePlanResponse

Plan = GetPlanResponsePlan

class PlansAPI:
    __client: PlansApi = None

    def __init__(self, vayu_client: VayuClient):
        vayu_client.validate_logged_in()
        self.__client = PlansApi(vayu_client.client)

    def list(self, limit: int = None, cursor: int = None):
        return self.__client.list_plans(limit=limit, cursor=cursor)

    def get(self, id: str):
        get_plans_response = self.__client.get_plan(plan_id=id)

        return get_plans_response

    def delete(self, id: str):
        delete_plan_response = self.__client.delete_plan(plan_id=id)

        return delete_plan_response




import os

from setuptools import find_packages, setup

from vayu_sdk import __version__


def read_requirements():
    requirements = []
    if os.path.exists('requirements.txt'):
        with open('requirements.txt') as req:
            requirements = req.read().splitlines()
    return requirements

setup(
    name="vayu-client",
    version=__version__,
    packages=find_packages(),
    install_requires=read_requirements(),
    python_requires=">=3.7",
    description="The Vayu API client library in Python",
    author="Vayu Team",
    author_email="team@withvayu.com",
    url="https://github.com/weft-finance/vayu-py",
)


# coding: utf-8

"""
    Vayu API

    The Vayu API is a RESTful API that allows you to submit events for processing and storage & manage billing related entities.           The API is secured using the Bearer Authentication scheme with JWT tokens.           To obtain a JWT token, please contact Vayu at team@withvayu.com

    The version of the OpenAPI document: 1.0.0
    Contact: dev@withvayu.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr, field_validator
from typing import Optional, Union
from typing_extensions import Annotated
from openapi.models.create_customer_request import CreateCustomerRequest
from openapi.models.create_customer_response import CreateCustomerResponse
from openapi.models.delete_customer_response import DeleteCustomerResponse
from openapi.models.get_customer_response import GetCustomerResponse
from openapi.models.list_customers_response import ListCustomersResponse
from openapi.models.update_customer_request import UpdateCustomerRequest
from openapi.models.update_customer_response import UpdateCustomerResponse

from openapi.api_client import ApiClient, RequestSerialized
from openapi.api_response import ApiResponse
from openapi.rest import RESTResponseType


class CustomersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def create_customer(
        self,
        create_customer_request: CreateCustomerRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CreateCustomerResponse:
        """Create Customer

        Create a new Customer.

        :param create_customer_request: (required)
        :type create_customer_request: CreateCustomerRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_customer_serialize(
            create_customer_request=create_customer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateCustomerResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def create_customer_with_http_info(
        self,
        create_customer_request: CreateCustomerRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CreateCustomerResponse]:
        """Create Customer

        Create a new Customer.

        :param create_customer_request: (required)
        :type create_customer_request: CreateCustomerRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_customer_serialize(
            create_customer_request=create_customer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateCustomerResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def create_customer_without_preload_content(
        self,
        create_customer_request: CreateCustomerRequest,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create Customer

        Create a new Customer.

        :param create_customer_request: (required)
        :type create_customer_request: CreateCustomerRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create_customer_serialize(
            create_customer_request=create_customer_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateCustomerResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create_customer_serialize(
        self,
        create_customer_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_customer_request is not None:
            _body_params = create_customer_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/customers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_customer(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> DeleteCustomerResponse:
        """Delete Customer

        Delete a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeleteCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_customer_with_http_info(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[DeleteCustomerResponse]:
        """Delete Customer

        Delete a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeleteCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_customer_without_preload_content(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete Customer

        Delete a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "DeleteCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_customer_serialize(
        self,
        customer_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if customer_id is not None:
            _path_params['customerId'] = customer_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/customers/{customerId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_customer(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCustomerResponse:
        """Get Customer

        Get a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_customer_with_http_info(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCustomerResponse]:
        """Get Customer

        Get a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_customer_without_preload_content(
        self,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Customer

        Get a Customer by id.

        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_serialize(
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_customer_serialize(
        self,
        customer_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if customer_id is not None:
            _path_params['customerId'] = customer_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/customers/{customerId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_customer_by_external_id(
        self,
        external_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> GetCustomerResponse:
        """Get customer by externalId

        Use this endpoint to get a specific customer using its external Id.

        :param external_id: (required)
        :type external_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_by_external_id_serialize(
            external_id=external_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_customer_by_external_id_with_http_info(
        self,
        external_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[GetCustomerResponse]:
        """Get customer by externalId

        Use this endpoint to get a specific customer using its external Id.

        :param external_id: (required)
        :type external_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_by_external_id_serialize(
            external_id=external_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_customer_by_external_id_without_preload_content(
        self,
        external_id: StrictStr,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get customer by externalId

        Use this endpoint to get a specific customer using its external Id.

        :param external_id: (required)
        :type external_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_customer_by_external_id_serialize(
            external_id=external_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "GetCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_customer_by_external_id_serialize(
        self,
        external_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if external_id is not None:
            _path_params['externalId'] = external_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/customers/externalId/{externalId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_customers(
        self,
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ListCustomersResponse:
        """List Customers

        Get a list of Customers.

        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_customers_serialize(
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCustomersResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_customers_with_http_info(
        self,
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ListCustomersResponse]:
        """List Customers

        Get a list of Customers.

        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_customers_serialize(
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCustomersResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_customers_without_preload_content(
        self,
        limit: Optional[Union[Annotated[float, Field(le=1000, strict=True)], Annotated[int, Field(le=1000, strict=True)]]] = None,
        cursor: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Customers

        Get a list of Customers.

        :param limit:
        :type limit: float
        :param cursor:
        :type cursor: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_customers_serialize(
            limit=limit,
            cursor=cursor,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ListCustomersResponse",
            '400': None,
            '401': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_customers_serialize(
        self,
        limit,
        cursor,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if cursor is not None:
            
            _query_params.append(('cursor', cursor))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/customers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_customer(
        self,
        update_customer_request: UpdateCustomerRequest,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> UpdateCustomerResponse:
        """Update Customer

        Update a Customer by id.

        :param update_customer_request: (required)
        :type update_customer_request: UpdateCustomerRequest
        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_customer_serialize(
            update_customer_request=update_customer_request,
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_customer_with_http_info(
        self,
        update_customer_request: UpdateCustomerRequest,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[UpdateCustomerResponse]:
        """Update Customer

        Update a Customer by id.

        :param update_customer_request: (required)
        :type update_customer_request: UpdateCustomerRequest
        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_customer_serialize(
            update_customer_request=update_customer_request,
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_customer_without_preload_content(
        self,
        update_customer_request: UpdateCustomerRequest,
        customer_id: Annotated[str, Field(strict=True)],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update Customer

        Update a Customer by id.

        :param update_customer_request: (required)
        :type update_customer_request: UpdateCustomerRequest
        :param customer_id: (required)
        :type customer_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_customer_serialize(
            update_customer_request=update_customer_request,
            customer_id=customer_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "UpdateCustomerResponse",
            '400': None,
            '401': None,
            '404': None,
            '429': None,
            '500': None,
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_customer_serialize(
        self,
        update_customer_request,
        customer_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, Union[str, bytes]] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if customer_id is not None:
            _path_params['customerId'] = customer_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if update_customer_request is not None:
            _body_params = update_customer_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'BearerAuthorizer'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/customers/{customerId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )


